@@ -3,6 +3,7 @@ use flate2::Status;
 use object::Object;
 use object::ObjectSection;
 use std::collections::HashMap;
+use std::collections::HashSet;
 use std::convert::TryFrom;
 use std::path::{Path, PathBuf};
 
@@ -254,7 +255,9 @@ fn read_mapping_regions(
             let raw_header = read_leb_usize(covfun, idx);
             let header = read_counter(raw_header); //read_leb_usize(covfun, idx)); // counter or pseudo-counter
             match header {
-                RawCounter::Zero if raw_header != 0 => return Err(ReadCovMapError::PseudoCountersNotSupportedYet),
+                RawCounter::Zero if raw_header != 0 => {
+                    return Err(ReadCovMapError::PseudoCountersNotSupportedYet { raw_header })
+                }
                 _ => {}
             }
             let delta_line_start = read_leb_usize(covfun, idx);
@@ -379,7 +382,6 @@ pub fn read_prf_data(prf_data: &[u8], idx: &mut usize) -> Result<Vec<PrfData>, R
     Ok(counts)
 }
 
-#[must_use]
 #[no_coverage]
 fn read_func_names(slice: &[u8], names: &mut Vec<String>) -> Result<(), ReadCovMapError> {
     let slices = slice.split(
@@ -447,14 +449,19 @@ pub fn process_function_records(
     let mut all_expressions = Vec::new();
     for function_counters in records {
         let mut expressions = vec![];
-        // let mut expressions = HashMap::<ExpandedExpression, MappingRegion>::new();
+        let mut expressions_set = HashSet::new();
         for (raw_counter, mapping_region) in function_counters.counters_list.iter() {
             let mut expanded = ExpandedExpression::default();
             expanded.push_counter(raw_counter, Sign::Positive, &function_counters);
-            expanded.sort(); // sort them so that their hash is the same if they are equal
-                             // expressions.insert(expanded, mapping_region.clone());
-            expressions.push((expanded, mapping_region.clone()));
+            expanded.sort(); // sort them to canonicalise their representation
+            if expressions_set.contains(&expanded) {
+                continue;
+            } else {
+                expressions_set.insert(expanded.clone());
+                expressions.push((expanded, mapping_region.clone()));
+            }
         }
+
         let name_function = (&prf_names[&function_counters.header.id.name_md5]).clone();
 
         let expressions = expressions.into_iter().collect::<Vec<_>>();
@@ -503,7 +510,9 @@ pub enum ReadCovMapError {
     CannotFindSection {
         section: CovMapSection,
     },
-    PseudoCountersNotSupportedYet,
+    PseudoCountersNotSupportedYet {
+        raw_header: usize,
+    },
     FailedToDecompress {
         section: CovMapSection,
         decompress_result: Result<Status, flate2::DecompressError>,
@@ -529,7 +538,7 @@ pub fn read_covmap(covmap: &[u8], idx: &mut usize) -> Result<CovMap, ReadCovMapE
         let length_encoded_data = read_i32(covmap, idx) as usize;
         let _always_0 = read_i32(covmap, idx);
         let version = read_i32(covmap, idx);
-        if version != 3 {
+        if (3..=5).contains(&version) == false {
             return Err(ReadCovMapError::InvalidVersion(version));
         }
 
@@ -635,7 +644,7 @@ pub struct RawFunctionCounters {
     pub counters_list: Vec<(RawCounter, MappingRegion)>,
 }
 
-#[derive(Clone, Debug, Default, PartialEq, Eq, Hash)]
+#[derive(Clone, Debug, Default, PartialEq, Eq, Hash, PartialOrd, Ord)]
 pub struct ExpandedExpression {
     pub add_terms: Vec<usize>, // Counter indices
     pub sub_terms: Vec<usize>, // Counter indices
@@ -807,7 +816,7 @@ impl Coverage {
                             continue;
                         } else if e.add_terms.len() == 1 && e.sub_terms.is_empty() {
                             single_counters.push(&mut slice[e.add_terms[0]] as *mut _);
-                        } else if e.add_terms.len() > 0 {
+                        } else if !e.add_terms.is_empty() {
                             expression_counters.push(e.optimised(slice));
                         } else {
                             panic!(
@@ -835,10 +844,11 @@ impl Coverage {
         all_self.drain_filter(
             #[no_coverage]
             |coverage| {
-                for filepath in &coverage.function_record.filenames {
-                    return !keep_f(filepath);
+                if let Some(filepath) = coverage.function_record.filenames.first() {
+                    !keep_f(filepath)
+                } else {
+                    false
                 }
-                return false;
             },
         );
     }
