@@ -2,6 +2,7 @@
 
 mod leb128;
 mod llvm_coverage;
+#[cfg(feature = "serde_json_serializer")]
 mod serialized;
 
 use crate::traits::{SaveToStatsFolder, Sensor};
@@ -64,7 +65,7 @@ impl CodeCoverageSensor {
 
         let mut coverage = unsafe { Coverage::new(covfun, prf_data, get_counters()) }
             .expect("failed to properly link the different LLVM coverage sections");
-        // coverage.drain_filter(|coverage| coverage.single_counters.len() + coverage.expression_counters.len() <= 1);
+        coverage.drain_filter(|coverage| coverage.single_counters.is_empty());
         Coverage::filter_function_by_files(&mut coverage, keep);
 
         let mut count_instrumented = 0;
@@ -109,7 +110,7 @@ impl Sensor for CodeCoverageSensor {
             let CodeCoverageSensor { coverage, .. } = self;
             let mut index = 0;
             for coverage in coverage {
-                let single = *coverage.single_counters.get_unchecked(0);
+                let single = coverage.single_counters[0];
                 if *single == 0 {
                     // that happens kind of a lot? not sure it is worth simplifying
                     index += coverage.single_counters.len() + coverage.expression_counters.len();
@@ -138,8 +139,14 @@ impl Sensor for CodeCoverageSensor {
 impl SaveToStatsFolder for CodeCoverageSensor {
     #[no_coverage]
     fn save_to_stats_folder(&self) -> Vec<(PathBuf, Vec<u8>)> {
-        let coverage_map = self.coverage_map();
-        let content = serde_json::to_vec(&coverage_map).unwrap();
-        vec![(PathBuf::new().join("coverage_sensor.json"), content)]
+        cfg_if::cfg_if! {
+            if #[cfg(feature = "serde_json_serializer")] {
+                let coverage_map = self.coverage_map();
+                let content = serde_json::to_vec(&coverage_map).unwrap();
+                vec![(PathBuf::new().join("coverage_sensor.json"), content)]
+            } else {
+                vec![]
+            }
+        }
     }
 }
