@@ -1,19 +1,9 @@
-//! Fuzzing engine. Connects the CodeCoverageSensor
-//!to the [Pool] and uses an evolutionary algorithm using [Mutator] to find new interesting
-//! test inputs.
-
-use crate::code_coverage_sensor::CodeCoverageSensor;
 use crate::data_structures::RcSlab;
-use crate::sensors_and_pools::stats::EmptyStats;
-use crate::sensors_and_pools::NoopSensor;
-use crate::sensors_and_pools::SimplestToActivateCounterPool;
-use crate::sensors_and_pools::UnitPool;
-use crate::sensors_and_pools::{AndPool, AndSensor};
-use crate::sensors_and_pools::{TestFailure, TestFailurePool, TestFailureSensor, TEST_FAILURE};
+use crate::sensors_and_pools::{
+    AndSensorAndPool, NoopSensor, TestFailure, TestFailurePool, TestFailureSensor, UnitPool, TEST_FAILURE,
+};
 use crate::signals_handler::set_signal_handlers;
-use crate::traits::SaveToStatsFolder;
-use crate::traits::{CompatibleWithSensor, CorpusDelta, Pool, Sensor};
-use crate::traits::{Mutator, Serializer};
+use crate::traits::{CorpusDelta, Mutator, SaveToStatsFolder, SensorAndPool, Serializer};
 use crate::world::World;
 use crate::{CSVField, FuzzedInput, ToCSV};
 use fuzzcheck_common::arg::{Arguments, FuzzerCommand};
@@ -35,6 +25,15 @@ static UPDATE_CORPUS_ERROR: &str = "the corpus could not be updated on the file
 
 static mut DID_FIND_ANY_TEST_FAILURE: bool = false;
 
+/// The result of a fuzz test, if it ends.
+///
+/// It contains two fields:
+/// 1. [`found_test_failure`](Self::found_test_failure) is `true` if the fuzzer found any failing test case
+/// 2. [`reason_for_stopping`](Self::reason_for_stopping) gives the reason why the fuzzer stopped.
+///
+/// If the fuzzer stopped because it found a failing test case, then `reason_for_stopping` has the
+/// value [`ReasonForStopping::TestFailure(T)`](crate::ReasonForStopping::TestFailure) where `T` is the
+/// failing test case.
 #[derive(Debug, Clone)]
 pub struct FuzzingResult<T> {
     pub found_test_failure: bool,
@@ -51,7 +50,8 @@ pub enum ReasonForStopping<T> {
 }
 
 /// The index to a test case in the fuzzer’s storage.
-#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash, serde::Serialize, serde::Deserialize)]
+#[cfg_attr(feature = "serde_json_serializer", derive(serde::Serialize, serde::Deserialize))]
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
 pub struct PoolStorageIndex(usize);
 
 #[cfg(test)]
@@ -67,14 +67,10 @@ enum FuzzerInputIndex<T> {
     Pool(PoolStorageIndex),
 }
 
-struct FuzzerState<T: Clone, M: Mutator<T>, Sens: Sensor, P: Pool>
-where
-    P: CompatibleWithSensor<Sens>,
-{
+struct FuzzerState<T: Clone, M: Mutator<T>> {
     mutator: M,
-    sensor: Sens,
+    sensor_and_pool: Box<dyn SensorAndPool>,
     pool_storage: RcSlab<FuzzedInput<T, M>>,
-    pool: P,
     /// The step given to the mutator when the fuzzer wants to create a new arbitrary test case
     arbitrary_step: M::ArbitraryStep,
     /// The index of the test case that is being tested
@@ -88,10 +84,7 @@ where
     world: World,
 }
 
-impl<T: Clone, M: Mutator<T>, Sens: Sensor, P: Pool> FuzzerState<T, M, Sens, P>
-where
-    P: CompatibleWithSensor<Sens>,
-{
+impl<T: Clone, M: Mutator<T>> FuzzerState<T, M> {
     #[no_coverage]
     fn get_input<'a>(
         fuzzer_input_idx: &'a FuzzerInputIndex<FuzzedInput<T, M>>,
@@ -118,23 +111,20 @@ fn update_fuzzer_stats(stats: &mut FuzzerStats, world: &mut World) {
     }
 }
 
-impl<T: Clone, M: Mutator<T>, Sens: Sensor, P: Pool> SaveToStatsFolder for FuzzerState<T, M, Sens, P>
+impl<T: Clone, M: Mutator<T>> SaveToStatsFolder for FuzzerState<T, M>
 where
-    P: CompatibleWithSensor<Sens>,
     Self: 'static,
 {
     #[no_coverage]
     fn save_to_stats_folder(&self) -> Vec<(std::path::PathBuf, Vec<u8>)> {
-        let mut contents = self.pool.save_to_stats_folder();
-        contents.extend(self.sensor.save_to_stats_folder());
+        let mut contents = self.sensor_and_pool.save_to_stats_folder();
         contents.extend(self.world.save_to_stats_folder());
         contents
     }
 }
 
-impl<T: Clone, M: Mutator<T>, Sens: Sensor, P: Pool> FuzzerState<T, M, Sens, P>
+impl<T: Clone, M: Mutator<T>> FuzzerState<T, M>
 where
-    P: CompatibleWithSensor<Sens>,
     Self: 'static,
 {
     #[no_coverage]
@@ -146,7 +136,7 @@ where
     fn receive_signal(&mut self, signal: i32) -> ! {
         self.world.report_event(
             FuzzerEvent::CaughtSignal(signal as i32),
-            Some((&self.fuzzer_stats, &self.pool.stats())),
+            Some((&self.fuzzer_stats, self.sensor_and_pool.stats().as_ref())),
         );
 
         match signal {
@@ -160,7 +150,7 @@ where
                 } else {
                     self.world.report_event(
                         FuzzerEvent::CrashNoInput,
-                        Some((&self.fuzzer_stats, &self.pool.stats())),
+                        Some((&self.fuzzer_stats, self.sensor_and_pool.stats().as_ref())),
                     );
                     exit(TerminationStatus::Crash as i32);
                 }
@@ -178,7 +168,8 @@ where
             .mutator
             .ordered_arbitrary(&mut self.arbitrary_step, self.settings.max_input_cplx)
         {
-            let (cache, step) = self.mutator.validate_value(&v).unwrap();
+            let cache = self.mutator.validate_value(&v).unwrap();
+            let step = self.mutator.default_mutation_step(&v, &cache);
             Some((FuzzedInput::new(v, cache, step, 0), cplx))
         } else {
             None
@@ -194,26 +185,20 @@ where
     }
 }
 
-pub struct Fuzzer<T, M, Sens, P>
+pub struct Fuzzer<T, M>
 where
     T: Clone,
     M: Mutator<T>,
-    Sens: Sensor,
-    P: Pool,
-    P: CompatibleWithSensor<Sens>,
     Self: 'static,
 {
-    state: FuzzerState<T, M, Sens, P>,
+    state: FuzzerState<T, M>,
     test: Box<dyn Fn(&T) -> bool>,
 }
 
-impl<T, M, Sens, P> Fuzzer<T, M, Sens, P>
+impl<T, M> Fuzzer<T, M>
 where
     T: Clone,
     M: Mutator<T>,
-    Sens: Sensor,
-    P: Pool,
-    P: CompatibleWithSensor<Sens>,
     Self: 'static,
 {
     #[no_coverage]
@@ -221,17 +206,15 @@ where
         test: Box<dyn Fn(&T) -> bool>,
         mutator: M,
         serializer: Box<dyn Serializer<Value = T>>,
-        sensor: Sens,
-        pool: P,
+        sensor_and_pool: Box<dyn SensorAndPool>,
         settings: Arguments,
         world: World,
     ) -> Self {
         let arbitrary_step = mutator.default_arbitrary_step();
         Fuzzer {
             state: FuzzerState {
-                sensor,
+                sensor_and_pool,
                 pool_storage: RcSlab::new(),
-                pool,
                 mutator,
                 arbitrary_step,
                 input_idx: FuzzerInputIndex::None,
@@ -250,9 +233,8 @@ where
             state:
                 FuzzerState {
                     mutator,
-                    sensor,
+                    sensor_and_pool,
                     pool_storage,
-                    pool,
                     input_idx,
                     fuzzer_stats,
                     serializer,
@@ -264,7 +246,7 @@ where
         } = self;
 
         // we have verified in the caller function that there is an input
-        let input = FuzzerState::<T, M, Sens, P>::get_input(input_idx, pool_storage).unwrap();
+        let input = FuzzerState::<T, M>::get_input(input_idx, pool_storage).unwrap();
 
         std::panic::set_hook(Box::new(
             #[no_coverage]
@@ -280,7 +262,7 @@ where
             },
         ));
 
-        sensor.start_recording();
+        sensor_and_pool.start_recording();
         let result = catch_unwind(AssertUnwindSafe(
             #[no_coverage]
             || (test)(input.value.borrow()),
@@ -306,7 +288,7 @@ where
                 DID_FIND_ANY_TEST_FAILURE = true;
             }
         }
-        sensor.stop_recording();
+        sensor_and_pool.stop_recording();
         if test_failure && self.state.settings.stop_after_first_failure {
             let serialized_input = serializer.to_data(&input.value);
             self.state
@@ -320,7 +302,7 @@ where
 
         let input_id = PoolStorageIndex(pool_storage.next_slot());
 
-        let deltas = pool.process(input_id, sensor, cplx);
+        let deltas = sensor_and_pool.process(input_id, cplx);
 
         if !deltas.is_empty() {
             let add_ref_count = deltas.iter().fold(
@@ -338,7 +320,7 @@ where
             world
                 .update_corpus(input_id, content, &deltas, serializer.extension())
                 .expect(UPDATE_CORPUS_ERROR);
-            world.report_event(event, Some((fuzzer_stats, &pool.stats())));
+            world.report_event(event, Some((fuzzer_stats, sensor_and_pool.stats().as_ref())));
             if add_ref_count > 0 {
                 let new_input = input.new_source(mutator);
                 // here I don't check the complexity of the new input,
@@ -361,14 +343,14 @@ where
     fn process_next_input(&mut self) -> Result<(), ReasonForStopping<T>> {
         let FuzzerState {
             pool_storage,
-            pool,
+            sensor_and_pool,
             input_idx,
             mutator,
             settings,
             ..
         } = &mut self.state;
         loop {
-            if let Some(idx) = pool.get_random_index() {
+            if let Some(idx) = sensor_and_pool.get_random_index() {
                 *input_idx = FuzzerInputIndex::Pool(idx);
                 let input = &mut pool_storage[idx.0];
                 let generation = input.generation;
@@ -386,8 +368,11 @@ where
 
                     break Ok(());
                 } else {
-                    pool.mark_test_case_as_dead_end(idx);
-                    continue;
+                    self.state.world.report_event(
+                        FuzzerEvent::End,
+                        Some((&self.state.fuzzer_stats, self.state.sensor_and_pool.stats().as_ref())),
+                    );
+                    break Err(ReasonForStopping::ExhaustedAllPossibleMutations);
                 }
             } else if let Some((input, cplx)) = self.state.arbitrary_input() {
                 self.state.input_idx = FuzzerInputIndex::Temporary(input);
@@ -400,7 +385,7 @@ where
             } else {
                 self.state.world.report_event(
                     FuzzerEvent::End,
-                    Some((&self.state.fuzzer_stats, &self.state.pool.stats())),
+                    Some((&self.state.fuzzer_stats, self.state.sensor_and_pool.stats().as_ref())),
                 );
                 break Err(ReasonForStopping::ExhaustedAllPossibleMutations);
             }
@@ -419,7 +404,8 @@ where
                 #[no_coverage]
                 |value| {
                     let value = self.state.serializer.from_data(&value)?;
-                    let (cache, mutation_step) = self.state.mutator.validate_value(&value)?;
+                    let cache = self.state.mutator.validate_value(&value)?;
+                    let mutation_step = self.state.mutator.default_mutation_step(&value, &cache);
                     Some(FuzzedInput::new(value, cache, mutation_step, 0))
                 },
             )
@@ -437,7 +423,7 @@ where
             #[no_coverage]
             |i| i.complexity(&self.state.mutator) > self.state.settings.max_input_cplx,
         );
-        assert!(!inputs.is_empty());
+        // assert!(!inputs.is_empty());
 
         self.state.world.set_checkpoint_instant();
         for input in inputs {
@@ -450,19 +436,21 @@ where
     }
 
     #[no_coverage]
-    fn main_loop(&mut self) -> Result<!, ReasonForStopping<T>> {
+    fn main_loop(&mut self, minify: bool) -> Result<!, ReasonForStopping<T>> {
         self.state.world.report_event(
             FuzzerEvent::Start,
-            Some((&self.state.fuzzer_stats, &self.state.pool.stats())),
-        );
-        self.process_initial_inputs()?;
-        self.state.world.report_event(
-            FuzzerEvent::DidReadCorpus,
-            Some((&self.state.fuzzer_stats, &self.state.pool.stats())),
+            Some((&self.state.fuzzer_stats, self.state.sensor_and_pool.stats().as_ref())),
         );
+        if !minify {
+            self.process_initial_inputs()?;
+            self.state.world.report_event(
+                FuzzerEvent::DidReadCorpus,
+                Some((&self.state.fuzzer_stats, self.state.sensor_and_pool.stats().as_ref())),
+            );
+        }
 
         self.state.world.set_checkpoint_instant();
-        let mut next_milestone = (self.state.fuzzer_stats.total_number_of_runs + 100_000) * 2;
+        let mut next_milestone = (self.state.fuzzer_stats.total_number_of_runs + 10) * 2;
         loop {
             let duration_since_beginning = self.state.world.elapsed_time_since_start();
             if duration_since_beginning > self.state.settings.maximum_duration {
@@ -476,7 +464,7 @@ where
                 update_fuzzer_stats(&mut self.state.fuzzer_stats, &mut self.state.world);
                 self.state.world.report_event(
                     FuzzerEvent::Pulse,
-                    Some((&self.state.fuzzer_stats, &self.state.pool.stats())),
+                    Some((&self.state.fuzzer_stats, self.state.sensor_and_pool.stats().as_ref())),
                 );
                 next_milestone = self.state.fuzzer_stats.total_number_of_runs * 2;
             }
@@ -492,42 +480,38 @@ pub enum TerminationStatus {
 }
 
 #[no_coverage]
-pub fn launch<T, M, Sens, P>(
+pub fn launch<T, M>(
     test: Box<dyn Fn(&T) -> bool>,
     mutator: M,
     serializer: Box<dyn Serializer<Value = T>>,
-    sensor: Sens,
-    pool: P,
+    sensor_and_pool: Box<dyn SensorAndPool>,
     mut args: Arguments,
 ) -> FuzzingResult<T>
 where
     T: Clone,
     M: Mutator<T>,
-    Fuzzer<T, M, CodeCoverageSensor, SimplestToActivateCounterPool>: 'static,
-    Sens: Sensor + 'static,
-    P: Pool + CompatibleWithSensor<Sens> + 'static,
+    Fuzzer<T, M>: 'static,
 {
     let command = &args.command;
     let reason_for_stopping = match command {
         FuzzerCommand::Fuzz => {
             if !args.stop_after_first_failure {
-                let test_failure = TestFailureSensor::default();
-                let sensor = AndSensor(sensor, test_failure);
+                let test_failure_sensor = TestFailureSensor::default();
                 let test_failure_pool = TestFailurePool::new("test_failures");
-                let pool = AndPool::new(pool, test_failure_pool, 254);
+                let sensor_and_pool =
+                    AndSensorAndPool::new(sensor_and_pool, Box::new((test_failure_sensor, test_failure_pool)), 254);
                 let mut fuzzer = Fuzzer::new(
                     test,
                     mutator,
                     serializer,
-                    sensor,
-                    pool,
+                    Box::new(sensor_and_pool),
                     args.clone(),
                     World::new(args.clone()).expect(WORLD_NEW_ERROR),
                 );
 
                 let mut stats_headers = vec![CSVField::String("time".to_string())];
                 stats_headers.extend(fuzzer.state.fuzzer_stats.csv_headers());
-                stats_headers.extend(fuzzer.state.pool.stats().csv_headers());
+                stats_headers.extend(fuzzer.state.sensor_and_pool.stats().csv_headers());
                 fuzzer
                     .state
                     .world
@@ -535,7 +519,7 @@ where
                     .expect(WRITE_STATS_ERROR);
                 unsafe { fuzzer.state.set_up_signal_handler() };
 
-                let reason_for_stopping = fuzzer.main_loop().unwrap_err();
+                let reason_for_stopping = fuzzer.main_loop(false).unwrap_err();
                 fuzzer.state.write_stats().expect(WRITE_STATS_ERROR);
 
                 reason_for_stopping
@@ -544,8 +528,7 @@ where
                     test,
                     mutator,
                     serializer,
-                    sensor,
-                    pool,
+                    sensor_and_pool,
                     args.clone(),
                     World::new(args.clone()).expect(WORLD_NEW_ERROR),
                 );
@@ -553,13 +536,13 @@ where
 
                 let mut stats_headers = vec![CSVField::String("time".to_string())];
                 stats_headers.extend(fuzzer.state.fuzzer_stats.csv_headers());
-                stats_headers.extend(fuzzer.state.pool.stats().csv_headers());
+                stats_headers.extend(fuzzer.state.sensor_and_pool.stats().csv_headers());
                 fuzzer
                     .state
                     .world
                     .append_stats_file(&stats_headers)
                     .expect(WRITE_STATS_ERROR);
-                let reason_for_stopping = fuzzer.main_loop().unwrap_err();
+                let reason_for_stopping = fuzzer.main_loop(false).unwrap_err();
                 fuzzer.state.write_stats().expect(WRITE_STATS_ERROR);
 
                 reason_for_stopping
@@ -569,14 +552,21 @@ where
             let world = World::new(args.clone()).expect(WORLD_NEW_ERROR);
             let value = world.read_input_file(input_file).expect(READ_INPUT_FILE_ERROR);
             let value = serializer.from_data(&value).expect(SERIALIZER_FROM_DATA_ERROR);
-            if let Some((cache, mutation_step)) = mutator.validate_value(&value) {
+            if let Some(cache) = mutator.validate_value(&value) {
+                let mutation_step = mutator.default_mutation_step(&value, &cache);
                 args.max_input_cplx = mutator.complexity(&value, &cache) - 0.01;
 
-                let sensor = AndSensor(sensor, NoopSensor);
-
+                let noop_sensor = NoopSensor;
                 let unit_pool = UnitPool::new(PoolStorageIndex(0));
-                let pool = AndPool::new(pool, unit_pool, 240);
-                let mut fuzzer = Fuzzer::new(test, mutator, serializer, sensor, pool, args.clone(), world);
+                let sensor_and_pool = AndSensorAndPool::new(sensor_and_pool, Box::new((noop_sensor, unit_pool)), 128);
+                let mut fuzzer = Fuzzer::new(
+                    test,
+                    mutator,
+                    serializer,
+                    Box::new(sensor_and_pool),
+                    args.clone(),
+                    world,
+                );
                 fuzzer
                     .state
                     .pool_storage
@@ -584,7 +574,7 @@ where
 
                 unsafe { fuzzer.state.set_up_signal_handler() };
 
-                fuzzer.main_loop().unwrap_err()
+                fuzzer.main_loop(true).unwrap_err()
             } else {
                 // TODO: send a better error message saying some inputs in the corpus cannot be read
                 // TODO: there should be an option to ignore invalid values
@@ -596,7 +586,8 @@ where
             let mut world = World::new(args.clone()).expect(WORLD_NEW_ERROR);
             let value = world.read_input_file(input_file).expect(READ_INPUT_FILE_ERROR);
             let value = serializer.from_data(&value).expect(SERIALIZER_FROM_DATA_ERROR);
-            if let Some((cache, mutation_step)) = mutator.validate_value(&value) {
+            if let Some(cache) = mutator.validate_value(&value) {
+                let mutation_step = mutator.default_mutation_step(&value, &cache);
                 let input = FuzzedInput::new(value, cache, mutation_step, 0);
                 let cplx = input.complexity(&mutator);
 
@@ -606,7 +597,7 @@ where
                 ));
 
                 if result.is_err() || !result.unwrap() {
-                    world.report_event::<EmptyStats>(FuzzerEvent::TestFailure, None);
+                    world.report_event(FuzzerEvent::TestFailure, None);
                     let content = serializer.to_data(&input.value);
                     world
                         .save_artifact(content, cplx, serializer.extension())
