@@ -6,7 +6,7 @@ use crate::mutators::either::Either;
 use crate::mutators::fixed_len_vector::{self, FixedLenVecMutator};
 use crate::mutators::grammar::mutators::{ASTMutator, ASTSingleVariant};
 use crate::mutators::incremental_map::IncrementalMapping;
-use crate::mutators::recursive::{RecurToMutator, RecursiveMutator};
+use crate::mutators::recursive::{RecurToMutator, RecursiveMutator, RecursiveMutatorUnmutateToken};
 use crate::mutators::tuples::Tuple1Mutator;
 use crate::mutators::vector::{UnmutateVecToken, VecMutator};
 use crate::mutators::Mutator;
@@ -64,7 +64,7 @@ impl<'a> ASTMappingSequence<'a> {
             .unwrap_or(*self.start_index);
 
         let v = &from_value[removal_idx];
-        let (new_s, new_c) = v.generate_string_starting_at_idx(start_index);
+        let (new_s, new_c) = v.generate_string_and_ast_map_starting_at_idx(start_index);
         let len = new_c.len;
 
         self.children.insert(removal_idx, new_c);
@@ -74,6 +74,7 @@ impl<'a> ASTMappingSequence<'a> {
         }
         *self.len += len;
     }
+    #[doc(hidden)]
     #[no_coverage]
     fn unmutate_value_from_element_token<M>(
         &mut self,
@@ -93,6 +94,7 @@ impl<'a> ASTMappingSequence<'a> {
         }
         *self.len = self.len.wrapping_add(diff_len);
     }
+    #[doc(hidden)]
     #[no_coverage]
     fn unmutate_value_from_insertion_token(
         &mut self,
@@ -110,7 +112,7 @@ impl<'a> ASTMappingSequence<'a> {
             .unwrap_or(*self.start_index);
 
         // I need to add the value v[insertion_idx] to s and c and update the following indices
-        let (new_s, new_c) = insert_element.generate_string_starting_at_idx(start_index);
+        let (new_s, new_c) = insert_element.generate_string_and_ast_map_starting_at_idx(start_index);
         let len = new_c.len;
         // adjust len of whole mapping
         // and indices of things following them
@@ -122,6 +124,7 @@ impl<'a> ASTMappingSequence<'a> {
             child.start_index += len;
         }
     }
+    #[doc(hidden)]
     #[no_coverage]
     fn unmutate_value_from_removal_token(&mut self, to_value: &mut String, removal_idx: usize) {
         let (start_idx, len) = {
@@ -193,7 +196,7 @@ where
                 let mut tmp_cs = vec![];
 
                 for v in vs {
-                    let (new_s, new_c) = v.generate_string_starting_at_idx(start_index);
+                    let (new_s, new_c) = v.generate_string_and_ast_map_starting_at_idx(start_index);
                     let len = new_c.len;
                     start_index += len;
 
@@ -242,7 +245,7 @@ where
                 self.children.clear();
                 let mut new_s = String::new();
                 for x in from_value {
-                    let c = x.generate_string_in(&mut new_s, &mut start_index);
+                    let c = x.generate_string_and_ast_map_in(&mut new_s, &mut start_index);
                     self.children.push(c);
                 }
                 to_value.replace_range(*self.start_index..*self.start_index + *self.len, &new_s);
@@ -276,6 +279,7 @@ where
         }
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn unmutate_value_from_token(&mut self, to_value: &mut String, token: &UnmutateVecToken<AST, M>) {
         match token {
@@ -341,7 +345,7 @@ where
                 let mut tmp_s = String::new();
                 let mut tmp_cs = vec![];
                 for x in xs {
-                    let (new_s, new_c) = x.generate_string_starting_at_idx(start_index);
+                    let (new_s, new_c) = x.generate_string_and_ast_map_starting_at_idx(start_index);
                     let len = new_c.len;
                     start_index += len;
                     total_len += len;
@@ -364,7 +368,7 @@ where
                 self.children.clear();
                 let mut new_s = String::new();
                 for x in x {
-                    let c = x.generate_string_in(&mut new_s, &mut start_index);
+                    let c = x.generate_string_and_ast_map_in(&mut new_s, &mut start_index);
                     self.children.push(c);
                 }
                 to_value.replace_range(*self.start_index..*self.start_index + *self.len, &new_s);
@@ -423,13 +427,26 @@ where
                 }
                 *self.len = self.len.wrapping_add(diff_len);
             }
+            fixed_len_vector::UnmutateVecToken::Elements(tokens) => {
+                for (idx, t) in tokens {
+                    let v = &from_value[*idx];
+                    let child = &mut self.children[*idx];
+                    let old_len = child.len;
+                    child.mutate_value_from_token(v, to_value, t);
+                    let diff_len = child.len.wrapping_sub(old_len);
+                    for child in &mut self.children[idx + 1..] {
+                        child.start_index = child.start_index.wrapping_add(diff_len);
+                    }
+                    *self.len = self.len.wrapping_add(diff_len);
+                }
+            }
             fixed_len_vector::UnmutateVecToken::Replace(_) => {
                 let mut start_index = *self.start_index;
                 let original_start_idx = start_index;
                 self.children.clear();
                 let mut new_s = String::new();
                 for x in from_value {
-                    let c = x.generate_string_in(&mut new_s, &mut start_index);
+                    let c = x.generate_string_and_ast_map_in(&mut new_s, &mut start_index);
                     self.children.push(c);
                 }
                 to_value.replace_range(*self.start_index..*self.start_index + *self.len, &new_s);
@@ -438,6 +455,7 @@ where
         }
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn unmutate_value_from_token(&mut self, to_value: &mut String, token: &fixed_len_vector::UnmutateVecToken<AST, M>) {
         match token {
@@ -451,13 +469,25 @@ where
                 }
                 *self.len = self.len.wrapping_add(diff_len);
             }
+            fixed_len_vector::UnmutateVecToken::Elements(tokens) => {
+                for (idx, t) in tokens {
+                    let child = &mut self.children[*idx];
+                    let old_len = child.len;
+                    child.unmutate_value_from_token(to_value, t);
+                    let diff_len = child.len.wrapping_sub(old_len);
+                    for child in &mut self.children[idx + 1..] {
+                        child.start_index = child.start_index.wrapping_add(diff_len);
+                    }
+                    *self.len = self.len.wrapping_add(diff_len);
+                }
+            }
             fixed_len_vector::UnmutateVecToken::Replace(x) => {
                 let mut start_index = *self.start_index;
                 let original_start_idx = start_index;
                 self.children.clear();
                 let mut new_s = String::new();
                 for x in x {
-                    let c = x.generate_string_in(&mut new_s, &mut start_index);
+                    let c = x.generate_string_and_ast_map_in(&mut new_s, &mut start_index);
                     self.children.push(c);
                 }
                 to_value.replace_range(*self.start_index..*self.start_index + *self.len, &new_s);
@@ -481,7 +511,7 @@ where
     ) {
         match token {
             alternation::UnmutateToken::Replace(_) => {
-                let (new_s, new_c) = from_value.generate_string_starting_at_idx(self.start_index);
+                let (new_s, new_c) = from_value.generate_string_and_ast_map_starting_at_idx(self.start_index);
                 to_value.replace_range(self.start_index..self.start_index + self.len, &new_s);
                 self.len = new_c.len;
                 self.content = new_c.content;
@@ -492,6 +522,7 @@ where
         }
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn unmutate_value_from_token(
         &mut self,
@@ -500,7 +531,7 @@ where
     ) {
         match token {
             alternation::UnmutateToken::Replace(x) => {
-                let (new_s, new_c) = x.generate_string_starting_at_idx(self.start_index);
+                let (new_s, new_c) = x.generate_string_and_ast_map_starting_at_idx(self.start_index);
                 to_value.replace_range(self.start_index..self.start_index + self.len, &new_s);
                 self.len = new_c.len;
                 self.content = new_c.content;
@@ -527,6 +558,7 @@ impl<'a> IncrementalMapping<char, String, CharWithinRangeMutator> for ASTMapping
         *self.len = from_value.len_utf8();
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn unmutate_value_from_token(
         &mut self,
@@ -599,6 +631,7 @@ where
         }
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn unmutate_value_from_token(
         &mut self,
@@ -633,6 +666,43 @@ where
     }
 }
 
+impl IncrementalMapping<AST, String, RecursiveMutator<ASTMutator>> for ASTMap {
+    #[no_coverage]
+    fn mutate_value_from_token(
+        &mut self,
+        from_value: &AST,
+        to_value: &mut String,
+        token: &<RecursiveMutator<ASTMutator> as Mutator<AST>>::UnmutateToken,
+    ) {
+        match token {
+            RecursiveMutatorUnmutateToken::Replace(_) => {
+                *to_value = from_value.to_string();
+            }
+            RecursiveMutatorUnmutateToken::Token(token) => {
+                <Self as IncrementalMapping<AST, String, ASTMutator>>::mutate_value_from_token(
+                    self, from_value, to_value, token,
+                )
+            }
+        }
+    }
+
+    #[no_coverage]
+    fn unmutate_value_from_token(
+        &mut self,
+        to_value: &mut String,
+        token: &<RecursiveMutator<ASTMutator> as Mutator<AST>>::UnmutateToken,
+    ) {
+        match token {
+            RecursiveMutatorUnmutateToken::Replace(ast) => {
+                *to_value = ast.to_string();
+            }
+            RecursiveMutatorUnmutateToken::Token(token) => {
+                <Self as IncrementalMapping<AST, String, ASTMutator>>::unmutate_value_from_token(self, to_value, token)
+            }
+        }
+    }
+}
+
 impl IncrementalMapping<AST, String, ASTMutator> for ASTMap {
     #[no_coverage]
     fn mutate_value_from_token(
@@ -659,6 +729,7 @@ impl IncrementalMapping<AST, String, ASTMutator> for ASTMap {
         >>::mutate_value_from_token(self, from_value, to_value, token.inner.as_ref());
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn unmutate_value_from_token(
         &mut self,
