@@ -39,7 +39,7 @@ type InnerASTMutator = Either<
 pub struct ASTMutator {
     inner: Box<InnerASTMutator>,
 }
-#[derive(Clone, Debug, PartialEq)]
+#[derive(Clone)]
 pub struct ASTMutatorCache {
     inner: Box<<InnerASTMutator as Mutator<AST>>::Cache>,
 }
@@ -49,7 +49,7 @@ impl ASTMutatorCache {
         Self { inner: Box::new(inner) }
     }
 }
-#[derive(Clone, Debug, PartialEq)]
+#[derive(Clone)]
 pub struct ASTMutatorMutationStep {
     inner: Box<<InnerASTMutator as Mutator<AST>>::MutationStep>,
 }
@@ -59,7 +59,7 @@ impl ASTMutatorMutationStep {
         Self { inner: Box::new(inner) }
     }
 }
-#[derive(Clone, Debug, PartialEq)]
+#[derive(Clone)]
 pub struct ASTMutatorArbitraryStep {
     inner: Box<<InnerASTMutator as Mutator<AST>>::ArbitraryStep>,
 }
@@ -72,7 +72,7 @@ impl ASTMutatorUnmutateToken {
         Self { inner: Box::new(inner) }
     }
 }
-#[derive(Clone, Debug, PartialEq)]
+#[derive(Clone)]
 pub struct ASTMutatorRecursingPartIndex {
     inner: Box<<InnerASTMutator as Mutator<AST>>::RecursingPartIndex>,
 }
@@ -96,9 +96,14 @@ impl Mutator<AST> for ASTMutator {
 
     #[doc(hidden)]
     #[no_coverage]
-    fn validate_value(&self, value: &AST) -> Option<(Self::Cache, Self::MutationStep)> {
-        let (cache, step) = self.inner.validate_value(value)?;
-        Some((Self::Cache::new(cache), Self::MutationStep::new(step)))
+    fn validate_value(&self, value: &AST) -> Option<Self::Cache> {
+        let cache = self.inner.validate_value(value)?;
+        Some(Self::Cache::new(cache))
+    }
+    #[doc(hidden)]
+    #[no_coverage]
+    fn default_mutation_step(&self, value: &AST, cache: &Self::Cache) -> Self::MutationStep {
+        Self::MutationStep::new(self.inner.default_mutation_step(value, &cache.inner))
     }
 
     #[doc(hidden)]
