@@ -22,7 +22,7 @@ use std::{fmt::Display, path::PathBuf};
 
 use crate::{
     fuzzer::PoolStorageIndex,
-    traits::{CompatibleWithSensor, CorpusDelta, Pool, SaveToStatsFolder, Sensor},
+    traits::{CompatibleWithSensor, CorpusDelta, Pool, SaveToStatsFolder, Sensor, SensorAndPool, Stats},
     CSVField, ToCSV,
 };
 
@@ -62,10 +62,6 @@ where
 {
     type Stats = AndPoolStats<P1::Stats, P2::Stats>;
 
-    #[no_coverage]
-    fn len(&self) -> usize {
-        self.p1.len() + self.p2.len()
-    }
     #[no_coverage]
     fn stats(&self) -> Self::Stats {
         AndPoolStats(self.p1.stats(), self.p2.stats())
@@ -78,20 +74,12 @@ where
             } else {
                 self.p2.get_random_index()
             }
+        } else if let Some(idx) = self.p2.get_random_index() {
+            Some(idx)
         } else {
-            if let Some(idx) = self.p2.get_random_index() {
-                Some(idx)
-            } else {
-                self.p1.get_random_index()
-            }
+            self.p1.get_random_index()
         }
     }
-
-    #[no_coverage]
-    fn mark_test_case_as_dead_end(&mut self, idx: PoolStorageIndex) {
-        self.p1.mark_test_case_as_dead_end(idx);
-        self.p2.mark_test_case_as_dead_end(idx);
-    }
 }
 impl<P1, P2> SaveToStatsFolder for AndPool<P1, P2>
 where
@@ -107,6 +95,15 @@ where
 }
 
 /// A sensor that combines two sensors
+///
+/// This type assumes nothing about the relationship between the two sensors.
+/// It is most likely that you are also using two different pools to process
+/// each sensor’s observations. Then, you can use an [`AndPool`] to combine these
+/// two pools and make them compatible with this `AndSensor`.
+///
+/// If you'd like to extend the observations of one sensor with the observations
+/// of another and keep using the same pool, you may want to look at
+/// [`MergedSensors`](crate::sensors_and_pools::MergedSensors) instead.
 pub struct AndSensor<S1, S2>(pub S1, pub S2)
 where
     S1: Sensor,
@@ -158,6 +155,12 @@ impl<S1: Display, S2: Display> Display for AndPoolStats<S1, S2> {
         write!(f, "{} {}", self.0, self.1)
     }
 }
+impl<S1: Display, S2: Display> Stats for AndPoolStats<S1, S2>
+where
+    S1: Stats,
+    S2: Stats,
+{
+}
 
 impl<S1, S2, P1, P2> CompatibleWithSensor<AndSensor<S1, S2>> for AndPool<P1, P2>
 where
@@ -249,3 +252,70 @@ where
         h
     }
 }
+
+/// Combines two [`SensorAndPool`](crate::traits::SensorAndPool) trait objects into one.
+pub struct AndSensorAndPool {
+    sap1: Box<dyn SensorAndPool>,
+    sap2: Box<dyn SensorAndPool>,
+    ratio_choose_first: u8,
+    rng: fastrand::Rng,
+}
+impl AndSensorAndPool {
+    #[no_coverage]
+    pub fn new(sap1: Box<dyn SensorAndPool>, sap2: Box<dyn SensorAndPool>, ratio_choose_first: u8) -> Self {
+        Self {
+            sap1,
+            sap2,
+            ratio_choose_first,
+            rng: fastrand::Rng::new(),
+        }
+    }
+}
+impl SaveToStatsFolder for AndSensorAndPool {
+    #[no_coverage]
+    fn save_to_stats_folder(&self) -> Vec<(PathBuf, Vec<u8>)> {
+        let mut x = self.sap1.save_to_stats_folder();
+        x.extend(self.sap2.save_to_stats_folder());
+        x
+    }
+}
+impl SensorAndPool for AndSensorAndPool {
+    #[no_coverage]
+    fn stats(&self) -> Box<dyn crate::traits::Stats> {
+        Box::new(AndPoolStats(self.sap1.stats(), self.sap2.stats()))
+    }
+
+    #[no_coverage]
+    fn start_recording(&mut self) {
+        self.sap1.start_recording();
+        self.sap2.start_recording();
+    }
+
+    #[no_coverage]
+    fn stop_recording(&mut self) {
+        self.sap1.stop_recording();
+        self.sap2.stop_recording();
+    }
+
+    #[no_coverage]
+    fn process(&mut self, input_id: PoolStorageIndex, cplx: f64) -> Vec<CorpusDelta> {
+        let mut x = self.sap1.process(input_id, cplx);
+        x.extend(self.sap2.process(input_id, cplx));
+        x
+    }
+
+    #[no_coverage]
+    fn get_random_index(&mut self) -> Option<PoolStorageIndex> {
+        if self.rng.u8(..) <= self.ratio_choose_first {
+            if let Some(idx) = self.sap1.get_random_index() {
+                Some(idx)
+            } else {
+                self.sap2.get_random_index()
+            }
+        } else if let Some(idx) = self.sap2.get_random_index() {
+            Some(idx)
+        } else {
+            self.sap1.get_random_index()
+        }
+    }
+}
