@@ -29,15 +29,13 @@ use super::compatible_with_iterator_sensor::CompatibleWithIteratorSensor;
 use crate::data_structures::{Slab, SlabKey};
 use crate::fenwick_tree::FenwickTree;
 use crate::fuzzer::PoolStorageIndex;
-use crate::traits::{CorpusDelta, Pool, SaveToStatsFolder};
+use crate::traits::{CorpusDelta, Pool, SaveToStatsFolder, Stats};
 use crate::{CSVField, ToCSV};
 use ahash::{AHashMap, AHashSet};
 use fastrand::Rng;
 use nu_ansi_term::Color;
-use std::cmp::Ordering;
 use std::fmt::Display;
 use std::hash::Hash;
-use std::ops::Range;
 use std::path::{Path, PathBuf};
 
 #[derive(Debug)]
@@ -48,7 +46,7 @@ impl Clone for CounterIdx {
     #[inline(always)]
     #[no_coverage]
     fn clone(&self) -> Self {
-        Self(self.0.clone())
+        Self(self.0)
     }
 }
 impl Copy for CounterIdx {}
@@ -279,13 +277,7 @@ impl SimplestToActivateCounterPool {
 
         let mut affected_counters = AHashSet::<CounterIdx>::new();
 
-        let deleted_values: Vec<_> = to_delete
-            .iter()
-            .map(
-                #[no_coverage]
-                |&key| key,
-            )
-            .collect();
+        let deleted_values: Vec<_> = to_delete.iter().copied().collect();
         let deleted_pool_storage_indices = deleted_values
             .iter()
             .map(
@@ -522,25 +514,9 @@ impl Pool for SimplestToActivateCounterPool {
         }
     }
 
-    #[no_coverage]
-    fn len(&self) -> usize {
-        self.slab_inputs.len()
-    }
-
     #[no_coverage]
     fn get_random_index(&mut self) -> Option<PoolStorageIndex> {
-        if self.ranked_inputs.len() == 0 {
-            return None;
-        }
-        let most = self.ranked_inputs.prefix_sum(self.ranked_inputs.len() - 1);
-        if most <= 0.0 {
-            return None;
-        }
-        let chosen_weight = gen_f64(&self.rng, 0.0..most);
-
-        // Find the first item which has a weight *higher* than the chosen weight.
-        let choice = self.ranked_inputs.first_index_past_prefix_sum(chosen_weight);
-
+        let choice = self.ranked_inputs.sample(&self.rng)?;
         let key = self.slab_inputs.get_nth_key(choice);
 
         let input = &mut self.slab_inputs[key];
@@ -552,79 +528,75 @@ impl Pool for SimplestToActivateCounterPool {
         self.ranked_inputs.update(choice, delta);
         Some(input.data)
     }
-
-    #[no_coverage]
-    fn mark_test_case_as_dead_end(&mut self, idx: PoolStorageIndex) {
-        for input_key in self.slab_inputs.keys() {
-            let input = &mut self.slab_inputs[input_key];
-            if input.data == idx {
-                input.score = 0.0;
-                break;
-            }
-        }
-        self.update_self_stats()
-    }
 }
 
 impl SaveToStatsFolder for SimplestToActivateCounterPool {
     #[no_coverage]
     fn save_to_stats_folder(&self) -> Vec<(PathBuf, Vec<u8>)> {
-        let path = PathBuf::new().join(format!("{}.json", &self.name));
+        cfg_if::cfg_if! {
+            if #[cfg(feature = "serde_json_serializer")]
+            {
+                let path = PathBuf::new().join(format!("{}.json", &self.name));
 
-        let all_hit_counters = self
-            .all_counters_ref
-            .iter()
-            .enumerate()
-            .filter(|(_, x)| x.least_complexity.is_some())
-            .map(|(idx, _)| idx)
-            .collect::<Vec<_>>();
+                let all_hit_counters = self
+                    .all_counters_ref
+                    .iter()
+                    .enumerate()
+                    .filter(|(_, x)| x.least_complexity.is_some())
+                    .map(|(idx, _)| idx)
+                    .collect::<Vec<_>>();
 
-        let best_for_counter = self
-            .all_counters_ref
-            .iter()
-            .enumerate()
-            .filter(|(_, x)| x.least_complexity.is_some())
-            .map(|(idx, _)| {
-                let f = &self.analysed_counters[&CounterIdx::new(idx)];
-                let key = f.least_complex_input;
-                let input = &self.slab_inputs[key].data;
-                (idx, *input)
-            })
-            .collect::<Vec<_>>();
+                let best_for_counter = self
+                    .all_counters_ref
+                    .iter()
+                    .enumerate()
+                    .filter(|(_, x)| x.least_complexity.is_some())
+                    .map(|(idx, _)| {
+                        let f = &self.analysed_counters[&CounterIdx::new(idx)];
+                        let key = f.least_complex_input;
+                        let input = &self.slab_inputs[key].data;
+                        (idx, *input)
+                    })
+                    .collect::<Vec<_>>();
 
-        let mut ranked_inputs = self
-            .slab_inputs
-            .keys()
-            .map(|key| {
-                let input = &self.slab_inputs[key];
-                (input.data, input.score)
-            })
-            .collect::<Vec<_>>();
-        ranked_inputs.sort_by(|x, y| x.1.partial_cmp(&y.1).unwrap_or(Ordering::Equal).reverse());
-        let ranked_inputs = ranked_inputs.into_iter().map(|x| x.0).collect();
+                let mut ranked_inputs = self
+                    .slab_inputs
+                    .keys()
+                    .map(|key| {
+                        let input = &self.slab_inputs[key];
+                        (input.data, input.score)
+                    })
+                    .collect::<Vec<_>>();
+                ranked_inputs.sort_by(|x, y| x.1.partial_cmp(&y.1).unwrap_or(std::cmp::Ordering::Equal).reverse());
+                let ranked_inputs = ranked_inputs.into_iter().map(|x| x.0).collect();
 
-        let counters_for_input = self
-            .slab_inputs
-            .keys()
-            .map(|key| {
-                let input = &self.slab_inputs[key];
-                (input.data, input.all_counters.iter().map(|x| x.0).collect())
-            })
-            .collect::<Vec<_>>();
+                let counters_for_input = self
+                    .slab_inputs
+                    .keys()
+                    .map(|key| {
+                        let input = &self.slab_inputs[key];
+                        (input.data, input.all_counters.iter().map(|x| x.0).collect())
+                    })
+                    .collect::<Vec<_>>();
 
-        let serialized = SerializedUniqCov {
-            all_hit_counters,
-            best_for_counter,
-            ranked_inputs,
-            counters_for_input,
-        };
+                let serialized = SerializedUniqCov {
+                    all_hit_counters,
+                    best_for_counter,
+                    ranked_inputs,
+                    counters_for_input,
+                };
 
-        let content = serde_json::to_vec(&serialized).unwrap();
+                let content = serde_json::to_vec(&serialized).unwrap();
 
-        vec![(path, content)]
+                vec![(path, content)]
+            } else {
+                vec![]
+            }
+        }
     }
 }
 
+#[cfg(feature = "serde_json_serializer")]
 #[derive(serde::Serialize, serde::Deserialize)]
 struct SerializedUniqCov {
     all_hit_counters: Vec<usize>,
@@ -633,12 +605,6 @@ struct SerializedUniqCov {
     counters_for_input: Vec<(PoolStorageIndex, Vec<usize>)>,
 }
 
-#[inline(always)]
-#[no_coverage]
-pub fn gen_f64(rng: &fastrand::Rng, range: Range<f64>) -> f64 {
-    range.start + rng.f64() * (range.end - range.start)
-}
-
 // ===============================================================
 // ==================== Trait implementations ====================
 // ===============================================================
@@ -695,7 +661,7 @@ impl ToCSV for UniqueCoveragePoolStats {
         ]
     }
 }
-
+impl Stats for UniqueCoveragePoolStats {}
 #[derive(Default)]
 pub struct UniqueCoveragePoolObservationState {
     is_interesting: bool,
@@ -899,9 +865,10 @@ mod tests {
         fn default_arbitrary_step(&self) -> Self::ArbitraryStep {}
 
         #[no_coverage]
-        fn validate_value(&self, _value: &f64) -> Option<(Self::Cache, Self::MutationStep)> {
-            Some(((), ()))
+        fn validate_value(&self, _value: &f64) -> Option<Self::Cache> {
+            Some(())
         }
+        fn default_mutation_step(&self, _value: &f64, _cache: &Self::Cache) -> Self::MutationStep {}
         #[no_coverage]
         fn max_complexity(&self) -> f64 {
             0.0
