@@ -2,17 +2,16 @@ use crate::data_structures::{Slab, SlabKey};
 use crate::fenwick_tree::FenwickTree;
 use crate::fuzzer::PoolStorageIndex;
 use crate::sensors_and_pools::compatible_with_iterator_sensor::CompatibleWithIteratorSensor;
-use crate::traits::{CorpusDelta, Pool, SaveToStatsFolder};
+use crate::traits::{CorpusDelta, Pool, SaveToStatsFolder, Stats};
 use crate::ToCSV;
 use ahash::{AHashMap, AHashSet};
 use std::fmt::{Debug, Display};
-use std::ops::Range;
 use std::path::Path;
 
 #[derive(Clone, Default)]
 pub struct UniqueValuesPoolStats {
-    name: String,
-    size: usize,
+    pub name: String,
+    pub size: usize,
 }
 impl ToCSV for UniqueValuesPoolStats {
     fn csv_headers(&self) -> Vec<crate::CSVField> {
@@ -34,6 +33,7 @@ impl Display for UniqueValuesPoolStats {
         )
     }
 }
+impl Stats for UniqueValuesPoolStats {}
 
 #[derive(Debug)]
 struct Input {
@@ -87,11 +87,6 @@ impl UniqueValuesPool {
 impl Pool for UniqueValuesPool {
     type Stats = UniqueValuesPoolStats;
 
-    #[no_coverage]
-    fn len(&self) -> usize {
-        self.inputs.len()
-    }
-
     #[no_coverage]
     fn stats(&self) -> Self::Stats {
         self.stats.clone()
@@ -99,15 +94,7 @@ impl Pool for UniqueValuesPool {
 
     #[no_coverage]
     fn get_random_index(&mut self) -> Option<PoolStorageIndex> {
-        if self.ranked_inputs.len() == 0 {
-            return None;
-        }
-        let most = self.ranked_inputs.prefix_sum(self.ranked_inputs.len() - 1);
-        if most <= 0.0 {
-            return None;
-        }
-        let chosen_weight = gen_f64(&self.rng, 0.0..most);
-        let choice = self.ranked_inputs.first_index_past_prefix_sum(chosen_weight);
+        let choice = self.ranked_inputs.sample(&self.rng)?;
         let key = self.inputs.get_nth_key(choice);
 
         let input = &mut self.inputs[key];
@@ -120,18 +107,6 @@ impl Pool for UniqueValuesPool {
         let data = self.inputs[key].data;
         Some(data)
     }
-
-    #[no_coverage]
-    fn mark_test_case_as_dead_end(&mut self, idx: PoolStorageIndex) {
-        for input_key in self.inputs.keys() {
-            let input = &mut self.inputs[input_key];
-            if input.data == idx {
-                input.score = 0.0;
-                break;
-            }
-        }
-        self.update_stats();
-    }
 }
 impl SaveToStatsFolder for UniqueValuesPool {
     #[no_coverage]
@@ -244,9 +219,3 @@ impl CompatibleWithIteratorSensor for UniqueValuesPool {
         }];
     }
 }
-
-#[inline(always)]
-#[no_coverage]
-fn gen_f64(rng: &fastrand::Rng, range: Range<f64>) -> f64 {
-    range.start + rng.f64() * (range.end - range.start)
-}
