@@ -109,7 +109,8 @@ use std::path::PathBuf;
 # use fuzzcheck::DefaultMutator;
 # let m = bool::default_mutator();
 # let mut value = false;
-# let (mut cache, mut step) = m.validate_value(&value).unwrap();
+# let mut cache = m.validate_value(&value).unwrap();
+# let mut step = m.default_mutation_step(&value, &cache);
 # let max_cplx = 8.0;
 # fn test(x: &bool) {}
 //  value = [[1, 3], [5], [9, 8]];
@@ -157,9 +158,12 @@ pub trait Mutator<Value: Clone>: 'static {
     fn default_arbitrary_step(&self) -> Self::ArbitraryStep;
 
     /// Verifies that the value conforms to the mutator’s expectations and, if it does,
-    /// returns the [`Cache`](Mutator::Cache) and first [`MutationStep`](Mutator::MutationStep)
-    /// associated with that value.
-    fn validate_value(&self, value: &Value) -> Option<(Self::Cache, Self::MutationStep)>;
+    /// returns the [`Cache`](Mutator::Cache) with that value.
+    fn validate_value(&self, value: &Value) -> Option<Self::Cache>;
+
+    /// Returns the first [`MutationStep`](Mutator::MutationStep) associated with the value
+    /// and cache.
+    fn default_mutation_step(&self, value: &Value, cache: &Self::Cache) -> Self::MutationStep;
 
     /// The maximum complexity that a value can possibly have.
     ///
@@ -304,10 +308,16 @@ where
 
     #[doc(hidden)]
     #[no_coverage]
-    fn validate_value(&self, value: &T) -> Option<(Self::Cache, Self::MutationStep)> {
+    fn validate_value(&self, value: &T) -> Option<Self::Cache> {
         self.wrapped_mutator().validate_value(value)
     }
 
+    #[doc(hidden)]
+    #[no_coverage]
+    fn default_mutation_step(&self, value: &T, cache: &Self::Cache) -> Self::MutationStep {
+        self.wrapped_mutator().default_mutation_step(value, cache)
+    }
+
     #[doc(hidden)]
     #[no_coverage]
     fn max_complexity(&self) -> f64 {
@@ -356,19 +366,19 @@ where
         self.wrapped_mutator().random_mutate(value, cache, max_cplx)
     }
 
+    #[doc(hidden)]
+    type RecursingPartIndex = W::RecursingPartIndex;
     #[doc(hidden)]
     #[no_coverage]
     fn unmutate(&self, value: &mut T, cache: &mut Self::Cache, t: Self::UnmutateToken) {
         self.wrapped_mutator().unmutate(value, cache, t)
     }
-
-    #[doc(hidden)]
-    type RecursingPartIndex = W::RecursingPartIndex;
     #[doc(hidden)]
     #[no_coverage]
     fn default_recursing_part_index(&self, value: &T, cache: &Self::Cache) -> Self::RecursingPartIndex {
         self.wrapped_mutator().default_recursing_part_index(value, cache)
     }
+
     #[doc(hidden)]
     #[no_coverage]
     fn recursing_part<'a, V, N>(&self, parent: &N, value: &'a T, index: &mut Self::RecursingPartIndex) -> Option<&'a V>
@@ -451,6 +461,67 @@ pub trait Sensor: SaveToStatsFolder {
     fn iterate_over_observations(&mut self, handler: Self::ObservationHandler<'_>);
 }
 
+pub trait Stats: Display + ToCSV + 'static {}
+
+/// An object safe trait that combines the methods of the [`Sensor`], [`Pool`], and [`CompatibleWithSensor`] traits.
+///
+/// While it's often useful to work with the [`Sensor`] and [`Pool`] traits separately, the
+/// fuzzer doesn't actually need to know about the sensor and pool individually. By having
+/// this `SensorAndPool` trait, we can give the fuzzer a `Box<dyn SensorAndPool>` and get rid of
+/// two generic type parameters: `S: Sensor` and `P: Pool + CompatibleWithSensor<S>`.
+///
+/// This is better for compile times and simplifies the implementation of the fuzzer. Users of
+/// `fuzzcheck` should feel free to ignore this trait, as it is arguably more an implementation detail
+/// than a fundamental building block of the fuzzer.
+///
+/// Currently, there are two types implementing `SensorAndPool`:
+/// 1. `(S, P)` where `S: Sensor` and `P: Pool + CompatibleWithSensor<S>`
+/// 2. [`AndSensorAndPool`](crate::sensors_and_pools::AndSensorAndPool)
+pub trait SensorAndPool: SaveToStatsFolder {
+    fn stats(&self) -> Box<dyn Stats>;
+    fn start_recording(&mut self);
+    fn stop_recording(&mut self);
+    fn process(&mut self, input_id: PoolStorageIndex, cplx: f64) -> Vec<CorpusDelta>;
+    fn get_random_index(&mut self) -> Option<PoolStorageIndex>;
+}
+impl<A, B> SaveToStatsFolder for (A, B)
+where
+    A: SaveToStatsFolder,
+    B: SaveToStatsFolder,
+{
+    fn save_to_stats_folder(&self) -> Vec<(PathBuf, Vec<u8>)> {
+        let mut x = self.0.save_to_stats_folder();
+        x.extend(self.1.save_to_stats_folder());
+        x
+    }
+}
+impl<S, P> SensorAndPool for (S, P)
+where
+    S: Sensor,
+    P: CompatibleWithSensor<S>,
+    S: SaveToStatsFolder,
+    P: SaveToStatsFolder,
+{
+    fn stats(&self) -> Box<dyn Stats> {
+        Box::new(self.1.stats())
+    }
+    fn start_recording(&mut self) {
+        self.0.start_recording();
+    }
+
+    fn stop_recording(&mut self) {
+        self.0.stop_recording();
+    }
+
+    fn process(&mut self, input_id: PoolStorageIndex, complexity: f64) -> Vec<CorpusDelta> {
+        self.1.process(input_id, &mut self.0, complexity)
+    }
+
+    fn get_random_index(&mut self) -> Option<PoolStorageIndex> {
+        self.1.get_random_index()
+    }
+}
+
 pub enum CSVField {
     Integer(isize),
     Float(f64),
@@ -499,22 +570,28 @@ pub trait ToCSV {
     /// by [self.csv_headers\()](ToCSV::csv_headers)
     fn to_csv_record(&self) -> Vec<CSVField>;
 }
+impl ToCSV for Box<dyn Stats> {
+    fn csv_headers(&self) -> Vec<CSVField> {
+        self.as_ref().csv_headers()
+    }
 
+    fn to_csv_record(&self) -> Vec<CSVField> {
+        self.as_ref().to_csv_record()
+    }
+}
+impl Stats for Box<dyn Stats> {}
 /**
-A [Pool] ranks test cases based on observations recorded by a sensor.
+A [`Pool`] ranks test cases based on observations recorded by a sensor.
 
 The pool trait is divided into two parts:
-1. [Pool] contains general methods that are independent of the sensor used
-2. [CompatibleWithSensor<Sensor>] is a subtrait of [Pool]. It describes how the pool handles
-observations made by the Sensor.
+1. [`Pool`] contains general methods that are independent of the sensor used
+2. [`CompatibleWithSensor<Sensor>`] is a subtrait of [`Pool`]. It describes how the pool handles
+observations made by the [`Sensor`].
 */
 pub trait Pool: SaveToStatsFolder {
     /// Statistics about the pool to be printed to the terminal as the fuzzer is running and
     /// saved to a .csv file after the run
-    type Stats: Display + ToCSV + Clone;
-
-    /// The number of test cases in the pool
-    fn len(&self) -> usize;
+    type Stats: Stats;
 
     /// The pool’s statistics
     fn stats(&self) -> Self::Stats;
@@ -524,12 +601,6 @@ pub trait Pool: SaveToStatsFolder {
     /// Most [Pool] implementations will want to prioritise certain test cases
     /// over others based on their associated observations.
     fn get_random_index(&mut self) -> Option<PoolStorageIndex>;
-
-    /// Mark a certain test case as a dead end.
-    ///
-    /// A test case is a dead end when its [Mutator] can no longer mutate it
-    /// to different values.
-    fn mark_test_case_as_dead_end(&mut self, idx: PoolStorageIndex);
 }
 
 /**
