@@ -1,8 +1,8 @@
 use crate::fuzzer::PoolStorageIndex;
 use crate::fuzzer::TerminationStatus;
-use crate::sensors_and_pools::stats::EmptyStats;
 use crate::traits::CorpusDelta;
 use crate::traits::SaveToStatsFolder;
+use crate::traits::Stats;
 use crate::CSVField;
 use crate::ToCSV;
 use fuzzcheck_common::arg::Arguments;
@@ -12,7 +12,6 @@ use nu_ansi_term::Color;
 use std::cell::RefCell;
 use std::collections::hash_map::DefaultHasher;
 use std::collections::HashMap;
-use std::fmt::Display;
 use std::fs;
 use std::fs::File;
 use std::fs::OpenOptions;
@@ -90,7 +89,7 @@ impl World {
     pub fn append_stats_file(&self, fields: &[CSVField]) -> Result<()> {
         if let Some(stats) = &self.stats {
             let mut stats = stats.try_borrow_mut().unwrap();
-            stats.write(&CSVField::to_bytes(fields))?;
+            stats.write_all(&CSVField::to_bytes(fields))?;
         }
         Ok(())
     }
@@ -107,13 +106,13 @@ impl World {
             let CorpusDelta { path, add, remove } = delta;
             for to_remove_key in remove {
                 let hash = self.corpus.remove(&(path.to_path_buf(), *to_remove_key)).unwrap();
-                self.remove_from_output_corpus(&path, hash.clone(), extension)?;
+                self.remove_from_output_corpus(path, hash.clone(), extension)?;
             }
 
             if *add {
                 let hash = self.hash(&content);
                 let _old = self.corpus.insert((path.to_path_buf(), idx), hash.clone());
-                self.add_to_output_corpus(&path, hash.clone(), content.clone(), extension)?;
+                self.add_to_output_corpus(path, hash.clone(), content.clone(), extension)?;
             }
         }
 
@@ -151,11 +150,7 @@ impl World {
     }
 
     #[no_coverage]
-    pub(crate) fn report_event<PoolStats: Display + ToCSV>(
-        &self,
-        event: FuzzerEvent,
-        stats: Option<(&FuzzerStats, &PoolStats)>,
-    ) {
+    pub(crate) fn report_event(&self, event: FuzzerEvent, stats: Option<(&FuzzerStats, &dyn Stats)>) {
         // println uses a lock, which may mess up the signal handling
         let time_since_start = self.initial_instant.elapsed();
         let time_since_start_display = {
@@ -322,7 +317,7 @@ This should never happen, and is probably a bug in fuzzcheck. Sorry :("#
 
     #[no_coverage]
     pub fn stop(&mut self) -> ! {
-        self.report_event::<EmptyStats>(FuzzerEvent::Stop, None);
+        self.report_event(FuzzerEvent::Stop, None);
         std::process::exit(TerminationStatus::Success as i32);
     }
 
@@ -340,7 +335,13 @@ This should never happen, and is probably a bug in fuzzcheck. Sorry :("#
 impl SaveToStatsFolder for World {
     #[no_coverage]
     fn save_to_stats_folder(&self) -> Vec<(PathBuf, Vec<u8>)> {
-        let content = serde_json::to_vec(&self.corpus.iter().collect::<Vec<_>>()).unwrap();
-        vec![(PathBuf::new().join("world.json"), content)]
+        cfg_if::cfg_if! {
+            if #[cfg(feature = "serde_json_serializer")] {
+                let content = serde_json::to_vec(&self.corpus.iter().collect::<Vec<_>>()).unwrap();
+                vec![(PathBuf::new().join("world.json"), content)]
+            } else {
+                vec![]
+            }
+        }
     }
 }
