@@ -6,11 +6,11 @@ use fuzzcheck::mutators::{
 
 fn test_alternation_char_helper(ranges: impl IntoIterator<Item = RangeInclusive<char>> + Clone) {
     let m = AlternationMutator::new(ranges.clone().into_iter().map(CharWithinRangeMutator::new).collect());
-    test_mutator(m, 100.0, 100.0, false, 100, 1000);
+    test_mutator(m, 100.0, 100.0, false, true, 100, 1000);
     let m = AlternationMutator::new(ranges.clone().into_iter().map(CharWithinRangeMutator::new).collect());
-    test_mutator(m, 1.0, 100.0, false, 100, 1000);
+    test_mutator(m, 1.0, 100.0, false, true, 100, 1000);
     let m = AlternationMutator::new(ranges.into_iter().map(CharWithinRangeMutator::new).collect());
-    test_mutator(m, 1.0, 1.0, false, 100, 1000);
+    test_mutator(m, 1.0, 1.0, false, true, 100, 1000);
 }
 
 #[test]
