@@ -12,7 +12,7 @@ struct SampleStruct<T, U> {
 #[test]
 fn test_derived_struct() {
     let mutator = SampleStruct::<u8, u8>::default_mutator();
-    test_mutator(mutator, 1000., 1000., false, 100, 100);
+    test_mutator(mutator, 1000., 1000., false, true, 100, 100);
     let mutator = <Vec<SampleStruct<u8, u8>>>::default_mutator();
-    test_mutator(mutator, 1000., 1000., false, 100, 100);
+    test_mutator(mutator, 1000., 1000., false, true, 100, 100);
 }
